package com.sun.org.glassfish.external.statistics.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;
import java.lang.reflect.Proxy;
import java.lang.reflect.InvocationHandler;
import com.sun.org.glassfish.external.statistics.RangeStatistic;

public final class RangeStatisticImpl extends StatisticImpl implements RangeStatistic, InvocationHandler
{
    private long currentVal;
    private long highWaterMark;
    private long lowWaterMark;
    private final long initCurrentVal;
    private final long initHighWaterMark;
    private final long initLowWaterMark;
    private final RangeStatistic rs;
    
    public RangeStatisticImpl(long curVal, long highMark, long lowMark, String name, String unit, String desc, long startTime, long sampleTime) {
        super(name, unit, desc, startTime, sampleTime);
        this.currentVal = 0L;
        this.highWaterMark = Long.MIN_VALUE;
        this.lowWaterMark = Long.MAX_VALUE;
        this.rs = (RangeStatistic)Proxy.newProxyInstance(RangeStatistic.class.getClassLoader(), new Class[] { RangeStatistic.class }, this);
        this.currentVal = curVal;
        this.initCurrentVal = curVal;
        this.highWaterMark = highMark;
        this.initHighWaterMark = highMark;
        this.lowWaterMark = lowMark;
        this.initLowWaterMark = lowMark;
    }
    
    public synchronized RangeStatistic getStatistic() {
        return this.rs;
    }
    
    @Override
    public synchronized Map getStaticAsMap() {
        Map<String, Long> m = super.getStaticAsMap();
        m.put("current", Long.valueOf(this.getCurrent()));
        m.put("lowwatermark", Long.valueOf(this.getLowWaterMark()));
        m.put("highwatermark", Long.valueOf(this.getHighWaterMark()));
        return m;
    }
    
    public synchronized long getCurrent() {
        return this.currentVal;
    }
    
    public synchronized void setCurrent(long curVal) {
        this.currentVal = curVal;
        this.lowWaterMark = ((curVal >= this.lowWaterMark) ? this.lowWaterMark : curVal);
        this.highWaterMark = ((curVal >= this.highWaterMark) ? curVal : this.highWaterMark);
        this.sampleTime = System.currentTimeMillis();
    }
    
    public synchronized long getHighWaterMark() {
        return this.highWaterMark;
    }
    
    public synchronized void setHighWaterMark(long hwm) {
        this.highWaterMark = hwm;
    }
    
    public synchronized long getLowWaterMark() {
        return this.lowWaterMark;
    }
    
    public synchronized void setLowWaterMark(long lwm) {
        this.lowWaterMark = lwm;
    }
    
    @Override
    public synchronized void reset() {
        super.reset();
        this.currentVal = this.initCurrentVal;
        this.highWaterMark = this.initHighWaterMark;
        this.lowWaterMark = this.initLowWaterMark;
        this.sampleTime = -1L;
    }
    
    @Override
    public synchronized String toString() {
        return super.toString() + RangeStatisticImpl.NEWLINE + "Current: " + this.getCurrent() + RangeStatisticImpl.NEWLINE + "LowWaterMark: " + this.getLowWaterMark() + RangeStatisticImpl.NEWLINE + "HighWaterMark: " + this.getHighWaterMark();
    }
    
    public Object invoke(Object proxy, Method m, Object[] args) throws Throwable {
        this.checkMethod(m);
        Object result;
        try {
            result = m.invoke(this, args);
        }
        catch (InvocationTargetException e) {
            throw e.getTargetException();
        }
        catch (Exception e2) {
            throw new RuntimeException("unexpected invocation exception: " + e2.getMessage());
        }
        return result;
    }
}
